check_recipe <- function(recipe, file, conf, logger) {
  # recipe: yaml recipe already read it
  # file: name of the file to clarify messages
  # conf: configuration file read to load the tables of definitions
  # output: errors or the total number of workflow (vars x regions) to compute
  info(logger, paste("Checking recipe", file))
  # create output dirs:
  if (!any(names(recipe) %in% "Analysis")) {
    error(logger, "The recipe should has a element 'Analysis'.")
  }
  params <- c('Horizon', 'Time', 'Variables', 'Region', 
              'Regrid', 'Workflow', 'Datasets')
  if (!all(params %in% names(recipe$Analysis))) {
    params <- paste(params, collapse = ' ')
    error(logger, 
          paste("The element 'Analysis' in the recipe should contain these",
         "elements:", params))
    stop("EXECUTION FAILED")
  }
  horizons <- c('Subseasonal', 'Seasonal', 'Decadal')
  if (!any(horizons %in% recipe$Analysis$Horizon)) {
    horizons <- paste(horizons, collapse = " ")
    error(logger,
         paste("The element 'Horizon' in the recipe",  
               "should be one of the followings:",
               horizons))
    stop("EXECUTION FAILED")
  }
  # Check temporal settings and
  # count the number of verifications
  time_settings <- c('sdate', 'leadtimemin', 'leadtimemax', 
                     'hcst_start', 'hcst_end')
  if (!all(time_settings %in% names(recipe$Analysis$Time))) {
    time_settings <- paste(time_settings, collapse = " ")
    error(logger, 
       paste("The element 'Time' in the recipe should contain these elements:",
             time_settings))
    stop("EXECUTION FAILED")
  }
  if (is.null(recipe$Analysis$Time$sdate$fcst_year) ||
    recipe$Analysis$Time$sdate$fcst_year == 'None') {
    stream <- "hindcast"
    recipe$Analysis$Time$sdate$fcst_year <- 'YYYY'
  } else {
    stream <- "fcst"
  }
  if (length(recipe$Analysis$Time$sdate$fcst_day) > 1 &&
      tolower(recipe$Analysis$Horizon) != "subseasonal") {
    warn(logger,
         paste("Only subseasonal verification allows multiple forecast days."),
               "Element fcst_day in recipe set as 1.")
    recipe$Analysis$Time$sdate$fcst_day <- '01'
  }
  if (is.null(recipe$Analysis$Time$sdate$fcst_month)) {
    error(logger,
          paste("The element 'fcst_month' in the recipe should be defined."))
  }
  fcst.sdate <- NULL
  for (year in recipe$Analysis$Time$sdate$fcst_year) {
    for (month in recipe$Analysis$Time$sdate$fcst_month) {
      for (day in recipe$Analysis$Time$sdate$fcst_day) {
        fcst.sdate <- c(fcst.sdate, 
                             paste0(year,
                               sprintf("%02d", as.numeric(month)),
                               sprintf("%02d", as.numeric(day))))
      }
    }
  }
  fcst.sdate <- list(stream = stream, fcst.sdate = fcst.sdate)   
  # Regrid checks:
  if (length(recipe$Analysis$Regrid) != 2) {
    error(logger,
          "The 'Regrid' element should specified the 'method' and 'type'.")
    stop("EXECUTION FAILED")    
  }
# more checks
  # ...
  # calculate number of workflows to create for each variable and 
  if (length(recipe$Analysis$Horizon) > 1) {
    error(logger, "Only 1 Horizon can be specified in the recipe")
    stop("EXECUTION FAILED")
  }
  nvar <- length(recipe$Analysis$Variables)
  if (nvar > 2) {
    error(logger,
          "Only two type of Variables can be listed: ECVs and Indicators.")
    stop("EXECUTION FAILED")
  }
  # remove NULL or None Indicators or ECVs from the recipe:
  if (!is.null(recipe$Analysis$Variables$Indicators) &&
      !is.list(recipe$Analysis$Variables$Indicators)) {
    recipe$Analysis$Variables <- recipe$Analysis$Variables[
           -which(names(recipe$Analysis$Variables) == 'Indicators')]
  }
  if (!is.null(recipe$Analysis$Variables$ECVs) &&
      !is.list(recipe$Analysis$Variables$ECVs)) {
    recipe$Analysis$Variables <- recipe$Analysis$Variables[
           -which(names(recipe$Analysis$Variables) == 'ECVs')]
  }
  # Only one Calibration method allowed:
  if ((is.logical(recipe$Analysis$Workflow$Calibration[[1]]) && 
                 recipe$Analysis$Workflow$Calibration[[1]] == FALSE) ||
                 recipe$Analysis$Workflow$Calibration[[1]] == 'None' ||
                 is.null(recipe$Analysis$Workflow$Calibration[[1]])) {
    warn(logger, 
         "There is no Calibration method selected, raw data verification.")
    recipe$Analysis$Workflow$Calibration[[1]] <- FALSE
  } else {
    # remove multiple calibration methods
    if (is.null(names(recipe$Analysis$Workflow$Calibration))) {
      error(logger,
          "The 'Calibration' element should specified at least the 'method'.")
      stop("EXECUTION FAILED")
    }
  }  
  if (length(recipe$Analysis$Region) > 3) {
    error(logger, "The section 'Region' in the recipe should be checked.",
         "Global, Countries and Regional elements are expected.")
    stop("EXECUTION FAILED")
  }
  if (!is.logical(recipe$Analysis$Region$Global)) {
    error(logger, "The element 'Global' in Region should be logical.")
    stop("EXECUTION FAILED")
  }
  nregions <- 0
  if (recipe$Analysis$Region$Global == TRUE) {
    nregions <- 1
  }
  if (recipe$Analysis$Region$Aggregation == TRUE) {
    nregions <- nregions + 1
  }
  if ("Regional" %in% names(recipe$Analysis$Region)) {
    nregions <- nregions + length(recipe$Analysis$Region$Regional)
    limits <- c('latmin', 'latmax', 'lonmin', 'lonmax')
    for (i in 1:length(recipe$Analysis$Region$Regional)) {
      if (!all(limits %in% names(recipe$Analysis$Region$Regional[[i]]))) {
        limits <- paste(limits, collapse = " ")
        error(logger,
           paste("Each region defined in element 'Regional'",
                 "should have 4 elements:",
                 limits))
        stop("EXECUTION FAILED")
      }
      # are numeric? class list mode list 
    }
  }
  # Check workflow: need to define restrictions? 
  # e.g. only one calibration method
  nverifications <- check_number_of_dependent_verifications(recipe, conf)
  info(logger, paste("Start Dates", paste(fcst.sdate, collapse = " ")))
  info(logger, "Recipe checked succsessfully.")
  return(append(nverifications, fcst.sdate))
}

check_number_of_dependent_verifications <- function(recipe, conf) {
  # Number of verifications depends on the variables and indicators requested
  # and the order of the workflow:
  # workflow: correction + indicator --> only 1 variable is calibrated
  # workflow: indicator + correction --> the indicator and the ecv are calibrated
  independent_verifications <- NULL
  dependent_verifications <- NULL
  dep <- 1
  # check workflow order:
  if (all(c('Calibration', 'Indicators') %in% names(recipe$Analysis$Workflow))) {
    cal_pos <- which(names(recipe$Analysis$Workflow) == 'Calibration')
    ind_pos <- which(names(recipe$Analysis$Workflow) == 'Indicators')
    if (cal_pos < ind_pos) {
      workflow_independent <- FALSE
    } else {
      workflow_independent <- TRUE
    }
  }
  if (workflow_independent) {
    independent_verifications <- append(recipe$Analysis$Variables$ECVs,
                                        recipe$Analysis$Variables$Indicators)
  } else { 
    if (is.null(recipe$Analysis$Variables$Indicators) || 
       (length(recipe$Analysis$Variables$Indicators) == 1 && 
        is.null(recipe$Analysis$Variables$ECVs))) {
      independent_verifications <- append(recipe$Analysis$Variables$ECVs,
                                          recipe$Analysis$Variables$Indicators)
    } else { 
      ecvs <- recipe$Analysi$Variables$ECVs
      inds <- recipe$Analysi$Variables$Indicators
      ind_table <- read_yaml(paste0(conf$code_dir,
                             "conf/indicators_table.yml"))
      # first, loop on ecvs if any and compare to indicators
      done <- NULL # to gather the indicators reviewed
      if (!is.null(ecvs)) {
        for (i in 1:length(ecvs)) {
          dependent <- list(ecvs[[i]])
          for (j in 1:length(inds)) {
            if (ind_table[inds[[j]]$name][[1]]$ECVs == ecvs[[i]]$name) {
              if (ind_table[inds[[j]]$name][[1]]$freq == ecvs[[i]]$freq) {
              # they are dependent
                dependent <- append(dependent, inds[[j]])
                done <- append(done, inds[[j]])
              }
            }
          }
          if (length(dependent) == 1) {
            dependent <- NULL
            independent_verifications <- append(independent_verifications,
                                              list(ecvs[[i]]))
          } else {
            dependent_verifications <- append(dependent_verifications,
                                              list(dependent))
          }  
        }
        # There are indicators not reviewed yet?
        if (length(done) < length(inds)) {
          if (length(inds) == 1) {
            independent_verifications <- append(independent_verifications,
                                                inds)
          } else {
            done <- NULL
            for (i in 1:(length(inds) - 1)) {
              dependent <- list(inds[[i]]$name)
              if (is.na(match(unlist(dependent), unlist(done)))) {
                for (j in (i+1):length(inds)) {
                  if (ind_table[inds[[i]]$name][[1]]$ECVs == 
                      ind_table[inds[[j]]$name][[1]]$ECVs) {
                    if (ind_table[inds[[i]]$name][[1]]$freq ==
                        ind_table[inds[[j]]$name][[1]]$freq) {
                      dependent <- append(dependent, inds[[j]]$name)
                      done <- dependent
                    }
                  }
                }
              } 
              if (length(dependent) == 1) {
                independent_verifications <- dependent
                dependent <- NULL     
              } else {
                dependent_verifications <- dependent
              }
            } 
          }
        }
      } else { # there are only Indicators:
        done <- NULL
        for (i in 1:(length(inds) - 1)) {
          dependent <- list(inds[[i]]$name)
          if (is.na(match(unlist(dependent), unlist(done)))) {
            for (j in (i+1):length(inds)) {
              if (ind_table[inds[[i]]$name][[1]]$ECVs ==
                  ind_table[inds[[j]]$name][[1]]$ECVs) {
                if (ind_table[inds[[i]]$name][[1]]$freq ==
                    ind_table[inds[[j]]$name][[1]]$freq) {
                  dependent <- append(dependent, inds[[j]]$name)
                  done <- dependent
                }
              }
            }
          }
          if (length(dependent) == 1) {
            independent_verifications <- dependent
            dependent <- NULL
          } else {
            dependent_verifications <- dependent
          }
        }
      } 
    }
  }
  if (!is.null(independent_verifications)) {   
    info(logger, paste("The variables for independent verification are ", 
         paste(independent_verifications, collapse = " ")))
  }
  if (!is.null(dependent_verifications)) {
    info(logger, paste("The variables for dependent verification are: ",
         paste(dependent_verifications, collapse = " ")))
  }
  # remove unnecessary names in objects to be removed
  return(list(independent = independent_verifications, 
              dependent = dependent_verifications))
}
#workflow <- list(Calibration = list(method = 'SBC'),
#                 Skill = list(metric = 'RPSS'))
#ApplyWorkflow <- function(workflow) {

#res <- do.call('CST_BiasCorrection',
#               args = list(exp = lonlat_data$exp,
#                           obs = lonlat_data$obs))
   



